@@ -658,7 +658,7 @@ draw_window (GdkDrawable        *drawable,
                       widget->style->bg_gc[GTK_STATE_NORMAL],
                       TRUE,
                       winrect->x + 1, winrect->y + 1,
-                      winrect->width - 2, winrect->height - 2);
+                      MAX (0, winrect->width - 2), MAX (0, winrect->height - 2));
 
   icon = wnck_window_get_icon (win);
 
@@ -731,7 +731,7 @@ draw_window (GdkDrawable        *drawable,
                       widget->style->fg_gc[GTK_STATE_NORMAL],
                       FALSE,
                       winrect->x, winrect->y,
-                      winrect->width - 1, winrect->height - 1);
+                      MAX (0, winrect->width - 1), MAX (0, winrect->height - 1));
 }            
 
 static int
@@ -771,6 +771,7 @@ workspace_at_point (WnckPager *pager,
   while (i < n_spaces)
     {
       GdkRectangle rect;
+      GtkWidget *widget;
       
       get_workspace_rect (pager, i, &rect);
 
@@ -779,7 +780,7 @@ workspace_at_point (WnckPager *pager,
        * Else, pretend the right/bottom line separating two workspaces belong
        * to the workspace.
        */
-      GtkWidget *widget = GTK_WIDGET (pager);
+      widget = GTK_WIDGET (pager);
 
       if (rect.x == xthickness)
         {
@@ -1261,12 +1262,14 @@ wnck_pager_button_release (GtkWidget        *widget,
 
 	  if (space)
             {
-              pager->priv->action_window = pager->priv->drag_window;
+              if (wnck_window_get_workspace (pager->priv->drag_window) != space)
+                pager->priv->action_window = pager->priv->drag_window;
+
               wnck_window_move_to_workspace (pager->priv->drag_window,
                                              space);
               if (space == wnck_screen_get_active_workspace (pager->priv->screen))
                 wnck_window_activate (pager->priv->drag_window, event->time);
-            }	  
+            }
 	}
       
       wnck_pager_clear_drag (pager);
