@@ -21,6 +21,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include "window.h"
 #include "class-group.h"
